'use strict';

exports.__esModule = true;
exports.tokenRegex = tokenRegex;
exports.parseToken = parseToken;
exports.serializeToken = serializeToken;

var _compare = require('./compare');

function tokenRegex() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return new RegExp('([\\(|-]+)?' + // capture prepended character, negation or paren
  '([\\w.]+)' + ( // the attribute name
  '' + (opts.partial ? '?' : '')) + ( // assume it's a token, even with no attribute
  ':' + (opts.partial ? '?' : '')) + // assume it's a token, even with no colon
  '([><=]*)' + // the operators
  '(?:(")(.*?)"|([^\\s()*:]*))' + ( // the attribute value, checking for quotes
  '' + (opts.noAttr || opts.partial ? '?' : '')) + // whether attribute value can be empty
  '(\\*)?', // capture appended wildcard
  'g');
}

function parseToken(value) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var results = Array.isArray(value) ? value : tokenRegex(opts).exec(value);

  if (!results || !results.length) {
    return {};
  }

  var tokenData = {
    fullToken: results[0],
    attributeName: results[2],
    attributeValue: results[5] || results[6],
    prepended: results[1] || '',
    operator: results[3],
    negated: results[0].indexOf('-') > -1,
    quoted: Boolean(results[4]),
    wildcard: Boolean(results[7])
  };

  var attributes = opts.attributes || [];
  var attribute = attributes.find(function (_ref) {
    var name = _ref.name;
    return (0, _compare.compareHard)(name, tokenData.attributeName);
  });

  if (attribute) {
    var hasEnums = attribute.enumerations && attribute.enumerations.length;

    tokenData.attributeNameValid = true;
    tokenData.attributeValueValid = hasEnums
    // do a soft check since we don't care about type
    ? attribute.enumerations.findIndex(function (v) {
      return (0, _compare.compareSoft)(tokenData.attributeValue, v);
    }) > -1
    // no enumerations, so any value is valid
    : true;
  }

  return tokenData;
}

function serializeToken(token) {
  var _token$prepended = token.prepended,
      prepended = _token$prepended === undefined ? '' : _token$prepended,
      _token$attributeName = token.attributeName,
      attributeName = _token$attributeName === undefined ? '' : _token$attributeName,
      _token$attributeValue = token.attributeValue,
      attributeValue = _token$attributeValue === undefined ? '' : _token$attributeValue,
      _token$operator = token.operator,
      operator = _token$operator === undefined ? '' : _token$operator;


  return '' + prepended + attributeName + ':' + operator + attributeValue;
}